<?php
/*
Plugin Name: EmailShroud
Plugin URI: http://www.somethinkodd.com/emailshroud/
Description: Prevents email address harvesting by replacing mailto: references in anchor tags with obfuscated form.
Version: 1.0.1
Author: Somethinkodd.com Development Team
Author URI: http://oddthinking.somethinkodd.com/
*/

/*  sto_emailShroud_mainFilter is the main engine to this plug-in.
    It replaces anchor tags referencing email addresses with Javascript
    that shrouds the email from harvesting. */

function stripLeftAndRight($paddedString) {
    return substr($paddedString, 1, strlen($paddedString)-2);
}
    
function sto_emailShroud_mainFilter($content) {
	$pattern  = <<<EMAILADDRESS_REGEXP
#(?:(?:mailto:)?([\w-]+(?:\.[\w-]+)*)@((?:[\w-]+\.)+\w+))|<a([^>]*)href\s*=\s*[\"\']?mailto:([^"'>\s@]*)@([^"'>\?\s]*)(\?[^\"\']*)?[\"\']?([^>]*)>([^<]+|.*?)?</a\s*>#i
EMAILADDRESS_REGEXP;
	/* 	Matches email addresses - here are some examples of matches:
			user.name@domain.name.tld
			mailto:user.name@domain.name.tld
			<a rel='external' href="mailto:user.name@domain.name.tld?subject=Subject Line" title="Tooltip">User's Full Name</a>
			
		Includes the following 'groups':
		1 : Simple address - user name
		2 : Simple address - domain name
		3 : Anchor tag - extra anchor tags between A and HREF (like rel='external nofollow')
		4 : Anchor tag - user name
		5 : Anchor tag - domain name
		6 : Anchor tag - extra fields of emailto (like "?subject=Test")
		7 : Anchor tag - extra anchor tags between HREF and end (like rel='external nofollow')
		8 : Anchor tag - subject - between <a> and </a>

		Note: When [1] is set, [4] is empty and vice-versa. Similarly for [2]/[5] and [1]/[8].
	*/
	return preg_replace_callback($pattern, 'sto_emailShroud_matchedEmailAddress', $content);
}

function sto_emailShroud_matchedEmailAddress($matchingText) {
/* Called by preg_replace_callback. See regexp in sto_emailShroud_mainFilter for definition of array. */

	$result = "<script type=\"text/javascript\"><!--\n";
	$result .= "\tsto_dom='$matchingText[2]$matchingText[5]'\n"; /* Only one of [2] and [5] will be non-null */
	$result .= "\tsto_user='$matchingText[1]$matchingText[4]'\n"; /* Only one of [1] and [4] will be non-null. */
	$result .= "\tdocument.write('<a $matchingText[3] href=\"mailto:' + sto_user + '@' +sto_dom + '$matchingText[6]\"";
	$result .= " $matchingText[7]>$matchingText[1]$matchingText[8]</a>')\n"; /* Only one of [1] and [8] will be non-null. */
	$result .= '//--></script>';

	$sto_emailShroud_actionPlan = get_option("sto_emailShroud_actionPlan");
	if ("replace" == $sto_emailShroud_actionPlan) {
		$sto_emailShroud_prefixString = stripLeftAndRight(get_option("sto_emailShroud_prefixString"));
		$sto_emailShroud_replacementString = stripLeftAndRight(get_option("sto_emailShroud_replacementString"));
		$sto_emailShroud_suffixString = stripLeftAndRight(get_option("sto_emailShroud_suffixString"));
		$result .= 
			"<noscript>"
			.$matchingText[1].$matchingText[8]
			.$sto_emailShroud_prefixString
			.$matchingText[1].$matchingText[4]
			.$sto_emailShroud_replacementString
			.$matchingText[2].$matchingText[5]
			.$sto_emailShroud_suffixString
			."</noscript>";
	} else {
		$sto_emailShroud_redirectURLbase = "http://www.somethinkodd.com/emailshroud/emailaddress.php";
		if("template" == $sto_emailShroud_actionPlan) {
			$sto_emailShroud_templateFile = get_option("sto_emailShroud_templateFile");
			$sto_emailShroud_redirectURLbase = get_bloginfo('template_url')."/".$sto_emailShroud_templateFile;
		}
		$result .=
			"<noscript>"
			."<a $matchingText[3] href=\"$sto_emailShroud_redirectURLbase?domainName=$matchingText[2]$matchingText[5]&amp;userName=$matchingText[1]$matchingText[4]\" $matchingText[7]>"
			."$matchingText[1]$matchingText[8]</a></noscript>";
	}
	return $result;
}

add_filter('the_content', 'sto_emailShroud_mainFilter', 55); /* Ensures that it is run after wp_texturize. Priority must be greater than 50 to work with Markdown 1.0.1b. */
add_filter('comment_text', 'sto_emailShroud_mainFilter', 55); /* Priority ensures that it is run after the cleanup which adds line breaks everywhere. */
add_filter('comment_excerpt', 'sto_emailShroud_mainFilter',55);
add_filter('the_excerpt', 'sto_emailShroud_mainFilter',55);
add_filter('the_excerpt_rss', 'sto_emailShroud_mainFilter',55);

function sto_emailShroud_options_page() {
    if (function_exists('add_options_page')) {
	add_options_page('EmailShroud', 'EmailShroud', 10, basename(__FILE__), 'sto_emailShroud_options_subpanel');
    }
}

add_action('admin_menu', 'sto_emailShroud_options_page');

function sto_emailShroud_options_subpanel() {
    if(get_option("sto_emailShroud_actionPlan")) {
        /* If one is set, blithely assume that all are set. */
	$sto_emailShroud_actionPlan = get_option("sto_emailShroud_actionPlan");
	$sto_emailShroud_templateFile = get_option("sto_emailShroud_templateFile");
	$sto_emailShroud_prefixString = stripLeftAndRight(get_option("sto_emailShroud_prefixString"));
	$sto_emailShroud_replacementString = stripLeftAndRight(get_option("sto_emailShroud_replacementString"));
	$sto_emailShroud_suffixString = stripLeftAndRight(get_option("sto_emailShroud_suffixString"));
    } else {
	$sto_emailShroud_actionPlan = "sto";
	$sto_emailShroud_templateFile = "emailAddress.php";
	$sto_emailShroud_prefixString = " [Email address: ";
	$sto_emailShroud_replacementString = " #AT# ";
	$sto_emailShroud_suffixString = " - replace #AT# with @ ]";
    };
    if (isset($_POST['info_update'])) {
        ?><div class="updated"><p><strong><?php 
        /* Note: There is insufficient checking of the data here.
            For example, the code should not ontain anything that will fool the preg_match script (e.g. $ signs).
            There should also not be any code that can't appear in a value tag (e.g. double-quote marks)
            While this may confuse the user, the risk of hostile attack is negligible. You need admin access to see this page. */
        if(isset($_POST['sto_emailShroud_actionPlan'])) {
           $sto_emailShroud_actionPlan = $_POST["sto_emailShroud_actionPlan"];
           update_option("sto_emailShroud_actionPlan", $sto_emailShroud_actionPlan);
           _e("Successfully set field: ", "emailShroud");
           echo "sto_emailShroud_actionPlan = '".$sto_emailShroud_actionPlan."'<br/>";
        };
        if(isset($_POST['sto_emailShroud_templateFile'])) {
            $sto_emailShroud_templateFile = $_POST["sto_emailShroud_templateFile"];
            update_option("sto_emailShroud_templateFile", $sto_emailShroud_templateFile);
           _e("Successfully set field: ", "emailShroud");
           echo "sto_emailShroud_templateFile = ".$sto_emailShroud_templateFile."<br/>";
        }
        if(isset($_POST['sto_emailShroud_prefixString'])) {
            $sto_emailShroud_prefixString = $_POST["sto_emailShroud_prefixString"];
            update_option("sto_emailShroud_prefixString", "~".$sto_emailShroud_prefixString."~");
            _e("Successfully set field: ", "emailShroud");
            echo "sto_emailShroud_prefixString = '".$sto_emailShroud_prefixString."'<br/>";
        }
        if(isset($_POST['sto_emailShroud_replacementString'])) {
           $sto_emailShroud_replacementString = $_POST["sto_emailShroud_replacementString"];
           update_option("sto_emailShroud_replacementString", "~".$sto_emailShroud_replacementString."~");
           _e("Successfully set field: ", "emailShroud");
           echo "sto_emailShroud_replacementString = ".$sto_emailShroud_replacementString."<br/>";
        }
        if(isset($_POST['sto_emailShroud_suffixString'])) {
           $sto_emailShroud_suffixString = $_POST["sto_emailShroud_suffixString"];
           update_option("sto_emailShroud_suffixString", "~".$sto_emailShroud_suffixString."~");
           _e("Successfully set field: ", "emailShroud");
           echo "sto_emailShroud_suffixString = ".$sto_emailShroud_suffixString."<br/>";
        }
        _e('Successfully updated.', 'emailShroud')?></strong></p></div><?php
    }
?><div class=wrap>
  <form method="post">
    <h2>EmailShroud Options</h2>
     <fieldset name="set1">
	<legend><?php _e('No JavaScript Action Plan', 'emailShroud'); ?></legend>
	<?php _e("If the reader's browser does not handle JavaScript, what action should be taken?", 'emailShroud'); ?>
	<ul>
		<li><input type="radio" <?php if("sto" == $sto_emailShroud_actionPlan){echo "CHECKED ";}?>name="sto_emailShroud_actionPlan" value="sto"/><?php _e("Divert to default decoder page on EmailShroud site.", 'emailShroud'); /* Note to potential translators: This site is in English only. Mention that in the translated version! */?></li>
		<li><input type="radio" <?php if("template" == $sto_emailShroud_actionPlan){echo "CHECKED ";}?>name="sto_emailShroud_actionPlan" value="template"/><?php _e("Divert to custom decoder page in the template file: ", 'emailShroud');?><input name="sto_emailShroud_templateFile" type="text" size="20" value="<?php echo $sto_emailShroud_templateFile; ?>"/></li>
		<li><input type="radio" <?php if("replace" == $sto_emailShroud_actionPlan){echo "CHECKED ";}?>name="sto_emailShroud_actionPlan" value="replace"/><?php _e("Transform address:", 'emailShroud'); ?>
                    <ul>
			<li><?php _e("Add prefix:", 'emailShroud');?><input name="sto_emailShroud_prefixString" type="text" size="30" value="<?php echo $sto_emailShroud_prefixString?>"/></li>
			<li><?php _e("Replace '@' with: ", 'emailShroud');?><input name="sto_emailShroud_replacementString" type="text" size="10" value="<?php echo $sto_emailShroud_replacementString?>"/></li>
			<li><?php _e("Add suffix:", 'emailShroud');?><input name="sto_emailShroud_suffixString" type="text" size="30" value="<?php echo $sto_emailShroud_suffixString?>"/></li>
                    </ul>
		</li>
	</ul>
     </fieldset>
<div class="submit">
  <input type="submit" name="info_update" value="<?php
    _e('Update options', 'Localization name');
	?> " /></div>
  </form>
 </div> <?php
}
?>
